/* hardware.c: code for changing hardware mode
   Copyright (c) 1998 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#include <stdio.h>

#include "snapconv.h"
#include "snapinfo.h"

#include "hardware.h"

int unsupportedHardware(struct snapinfo *sptr, hardwareMap *map, const char *format)
{
  int i=0;

  while(1)
    {
      if(verbose>=2)
	fprintf(stderr,"Applying map { %d, %d, %d, %d, %d }\n",
		map[i].machine,map[i].periph,map[i].newMachine,
		map[i].periphOn,map[i].periphOff);

      if(map[i].machine==UNSUPPORTED_LAST)
	{
	  break;
	}

      else if(map[i].machine==UNSUPPORTED_SPECIAL)
	{
	  switch(map[i].newMachine)
	    {
	      case UNSUPPORTED_SPECIAL128K:
		if(sptr->machine!=MACHINE_48K)
		  {
		    sptr->machine=MACHINE_48K;
		    printf(map[i].message,format);
		  }
		break;

	      case UNSUPPORTED_SPECIALNEW:
		if(sptr->machine!=MACHINE_48K && sptr->machine!=MACHINE_128K)
		  {
		    sptr->machine=MACHINE_128K;
		    printf(map[i].message,format);
		  }
		break;

	      case UNSUPPORTED_SPECIALPERIPH:
		if(sptr->periph)
		  {
		    sptr->periph=0;
		    printf(map[i].message,format);
		  }
		break;

	      default: return ERROR_UNKNOWNSPECIALMAP;
	    }
	}

      else if ( ( (map[i].machine==UNSUPPORTED_ANY) ||
		  (map[i].machine==sptr->machine)     ) &&
		( (map[i].periph&sptr->periph) == map[i].periph) )
	{
	  if(map[i].newMachine!=UNSUPPORTED_NOCHANGE)
	    {
	      sptr->machine=map[i].newMachine;
	    }
	  sptr->periph|=map[i].periphOn;
	  sptr->periph&=~map[i].periphOff;
	  printf(map[i].message,format);
	}

      i++; /* Select next map */

    }

  return ERROR_NONE;

}

/* Define some special maps */

hardwareMap UNSUPPORTED_48KNOPERIPHS[] = {

  UNSUPPORTED_128KMACHINES,
  UNSUPPORTED_PERIPHS,
  UNSUPPORTED_END

};


hardwareMap UNSUPPORTED_128KNOPERIPHS[] = {

  UNSUPPORTED_NEWMACHINES,
  UNSUPPORTED_PERIPHS,
  UNSUPPORTED_END

};
