/* hardware.h: code for changing hardware mode
   Copyright (c) 1998 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#ifndef hardware_h
#define hardware_h

#include <limits.h>

#include "snapinfo.h"

typedef struct hardwareMap {

  unsigned char machine; /* Map applies to this machine */
  unsigned short periph; /* with these peripherals active */

  unsigned char newMachine; /* Change to this machine */
  unsigned short periphOn; /* Turn these peripherals on */
  unsigned short periphOff; /* And these peripherals off */

  const char *message; /* Message to print when applied */

} hardwareMap;

int unsupportedHardware(struct snapinfo *sptr, hardwareMap *map, const char *format);

enum {

  UNSUPPORTED_LAST         = UCHAR_MAX   , /* Last entry in the map */
  UNSUPPORTED_ANY          =(UCHAR_MAX-1), /* Applies to any machine */
  UNSUPPORTED_SPECIAL      =(UCHAR_MAX-2), /* Special map */

  UNSUPPORTED_NOCHANGE     = UCHAR_MAX   , /* Don't change machine */

  UNSUPPORTED_SPECIAL128K  = UCHAR_MAX   , /* 48k only */
  UNSUPPORTED_SPECIALNEW   =(UCHAR_MAX-1), /* 128k only */
  UNSUPPORTED_SPECIALPERIPH=(UCHAR_MAX-2)  /* No peripherals */

};

/* Define some common map entries */

/* *Must* be the last entry in *every* map */

#define UNSUPPORTED_END { UNSUPPORTED_LAST, 0, 0, 0, 0, 0 }

/* Change to 48K */

#define UNSUPPORTED_128KMACHINES {\
\
  UNSUPPORTED_SPECIAL    , 0,\
  UNSUPPORTED_SPECIAL128K, 0, 0,\
  "Warning: %s does not support 128K snaps; Changing to 48K\n"\
\
}

/* Change to 48K/128K */

#define UNSUPPORTED_NEWMACHINES {\
\
  UNSUPPORTED_SPECIAL   , 0,\
  UNSUPPORTED_SPECIALNEW, 0, 0,\
  "Warning: %s does not support machines other than 48K/128K;\n"\
  "         Changing to 128K\n"\
\
}

/* Turn off all peripherals */

#define UNSUPPORTED_PERIPHS {\
\
  UNSUPPORTED_SPECIAL      , 0,\
  UNSUPPORTED_SPECIALPERIPH, 0, 0,\
  "Warning: %s does not support any additional peripherals;\n"\
  "         all disabled\n"\
\
}

/* Some common maps */

extern hardwareMap UNSUPPORTED_48KNOPERIPHS[]; /* 48K & no peripherals */
extern hardwareMap UNSUPPORTED_128KNOPERIPHS[]; /* 128K & no peripherals */

#endif			/* #ifndef hardware_h */
