/* plusd.c: routines for dealing with +D snapshots
   Copyright (c) 1998 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#include <stdio.h>
#include <string.h>

#include "snapconv.h"
#include "snapinfo.h"

#include "plusd.h"

int plusd_read(FILE *f,struct snapinfo *sptr,unsigned char *buffer)
{
  unsigned char Header[plusd_headerSize],*ptr;

  if(fread(Header,plusd_headerSize,1,f)!=1) return ERROR_READ;

  sptr->iy =Header[ 0]+Header[ 1]*0x100;
  sptr->ix =Header[ 2]+Header[ 3]*0x100;
  sptr->de_=Header[ 4]+Header[ 5]*0x100;
  sptr->bc_=Header[ 6]+Header[ 7]*0x100;
  sptr->hl_=Header[ 8]+Header[ 9]*0x100;
  sptr->f_ =Header[10]; sptr->a_=Header[11];
  sptr->de=Header[12]+Header[13]*0x100;
  sptr->bc=Header[14]+Header[15]*0x100;
  sptr->hl=Header[16]+Header[17]*0x100;
  sptr->i=Header[18]; /* Header offset 19 is `rubbish' */

  sptr->im = (sptr->i==0 || sptr->i==63) ? 1 : 2;

  sptr->sp=Header[20]+Header[21]*0x100;

  if(verbose>=2)
    {
      fprintf(stderr,"IY =0x%04x IX =0x%04x\n",sptr->iy,sptr->ix);
      fprintf(stderr,"DE'=0x%04x BC'=0x%04x\n",sptr->de_,sptr->bc_);
      fprintf(stderr,"HL'=0x%04x AF'=0x%02x%02x\n",
	      sptr->hl_,sptr->a_,sptr->f_);
      fprintf(stderr,"DE =0x%04x BC =0x%04x\n",sptr->de,sptr->bc);
      fprintf(stderr,"HL =0x%04x\n",sptr->hl);
      fprintf(stderr," I =0x%02x => IM %d\n",sptr->i,sptr->im);
      fprintf(stderr,"SP =0x%04x (Needs correcting)\n",sptr->sp);
    }

  /* Read in the memory contents */
  if(fread(buffer,0xc000,1,f)!=1) return ERROR_READ;

  /* R, IFF, AF and PC are stored on the stack */
  ptr=&buffer[sptr->sp-0x4000];

  sptr->r=ptr[0];
  sptr->iff1=(ptr[1]&0x04) >> 2; sptr->iff2=sptr->iff1;
  sptr->f=ptr[2]; sptr->a=ptr[3];
  sptr->pc=ptr[4]+ptr[5]*0x100;

  /* Zero corrupted bytes */
  memset(ptr,0,6);

  /* Increment SP */
  (sptr->sp)+=6;

  if(verbose>=2)
    {
      fprintf(stderr," R =0x%02x\n",sptr->r);
      fprintf(stderr,"IFF1=%d IFF2=%d\n",sptr->iff1,sptr->iff2);
      fprintf(stderr,"AF =0x%02x%02x\n",sptr->a,sptr->f);
      fprintf(stderr,"PC =0x%04x Correct SP=0x%04x\n",sptr->pc,sptr->sp);
    }

  return ERROR_NONE;

}

int plusd_write(FILE *f,struct snapinfo *sptr,unsigned char *buffer)
{
  unsigned int offset=(sptr->sp)-0x4000-6;

  writeInt(sptr->iy,f);  writeInt(sptr->ix,f);
  writeInt(sptr->de_,f); writeInt(sptr->bc_,f);
  writeInt(sptr->hl_,f); fputc(sptr->f_,f); fputc(sptr->a_,f);
  writeInt(sptr->de,f);  writeInt(sptr->bc,f);
  writeInt(sptr->hl,f);  fputc(sptr->i,f); fputc(0,f); /* `rubbish' */
  writeInt(sptr->sp-6,f); /* 6 to account for registers being stacked */

  /* Write out memory before variables being stacked */
  (void)fwrite(buffer,offset,1,f);

  /* Write out R, IFF, AF, PC */
  fputc(sptr->r,f); fputc( (sptr->iff2)<<2 , f);
  fputc(sptr->f,f); fputc(sptr->a,f);
  writeInt(sptr->pc,f);

  /* And then the rest of memory */
  (void)fwrite(&buffer[offset+6],0xc000-offset-6,1,f);

  return ferror(f) ? ERROR_WRITE : ERROR_NONE;

}
