/* snapconv.h: constants for snapconv.c, z80.c
   Copyright (c) 1997 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#ifndef snapconv_h
#define snapconv_h

#include <stdio.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

void writeInt(int i,FILE *f);

/* The number of RAM pages we need to store */

#define NUMRAMPAGES 8

typedef struct romData {

  int present; /* Is this ROM in the snapshot file? */
  unsigned char *ptr; /* Non-standard ROM data */

} romData;

/* The number of ROM pages we need to store */

#define NUMROMPAGES 15

enum {

  ROM_48K      = 0, /* The standard 48K ROM */
  ROM_128K_0   = 1, /* 128K ROM 0 (128K editor/menu system) */
  ROM_128K_1   = 2, /* 128K ROM 1 (48K BASIC) */
  ROM_PLUS2_0  = 3, /* +2 ROM 0 (128K editor/menu system) */
  ROM_PLUS2_1  = 4, /* +2 ROM 1 (48K BASIC) */
  ROM_PLUS3_0  = 5, /* +2A/+3 ROM 0 (128K editor/menu system/self-test) */
  ROM_PLUS3_1  = 6, /* +2A/+3 ROM 1 (128K syntax checker) */
  ROM_PLUS3_2  = 7, /* +2A/+3 ROM 2 (+3DOS) */
  ROM_PLUS3_3  = 8, /* +2A/+3 ROM 3 (48K BASIC) */
  ROM_IF1      = 9, /* Interface 1 ROM */
  ROM_DISCIPLE =10, /* Disciple ROM */
  ROM_PLUSD    =11, /* +D ROM */
  ROM_SAMRAM_0 =12, /* SamRam ROM 0 (BASIC) */
  ROM_SAMRAM_1 =13, /* SamRam ROM 1 (Monitor, etc) */
  ROM_MULTIFACE=14  /* Multiface ROM */

};

/* The length and checksum of each of the ROMs */

typedef struct romInfoStruct {

  unsigned long length; /* The length of the ROM */
  unsigned long checksum; /* The Adler-32 checksum of the ROM */

} romInfoStruct;

extern romInfoStruct romInfo[];

/* The types of error which can occur */

enum {

  ERROR_NONE	         = 0,
  ERROR_COMMANDLINE      = 1,
  ERROR_FILENAMES        = 2,
  ERROR_NOINFORMAT       = 3,
  ERROR_NOOUTFORMAT      = 4,
  ERROR_UNKNOWNINFORMAT  = 5,
  ERROR_UNKNOWNOUTFORMAT = 6,
  ERROR_OUTVERSION       = 7,
  ERROR_INOPEN           = 8,
  ERROR_OUTOPEN          = 9,
  ERROR_MEMORY           =10,
  ERROR_UNKNOWNPAGE      =11,
  ERROR_DUPLICATEPAGE    =12,
  ERROR_SNALENGTH	 =13,
  ERROR_READ	         =14,
  ERROR_WRITE	         =15,
  ERROR_CORRUPT	         =16,
  ERROR_UNKNOWNSPECIALMAP=17,
  ERROR_UNKNOWNHEADERLEN =18,
  ERROR_UNKNOWNHARDWARE  =19,
  ERROR_SUBCHUNKLENGTHS  =20,
  ERROR_PAGELENGTH       =21,
  ERROR_COMPRESSED       =22,
  ERROR_UNKNOWNCHUNK     =23,
  ERROR_NOFORMTYPE       =24,
  ERROR_COMPRESSION      =25,
  ERROR_CANTREAD         =26

};

extern int verbose;	/* How many messages to produce */

#ifndef HAVE_STRCASECMP
int strcasecmp (const char *S1, const char *S2);
#endif /* #ifndef HAVE_STRCASECMP */

#endif			/* #ifndef snapconv_h */
