/* snapinfo.h: defines struct snapinfo used by snapconv
   Copyright (c) 1997 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#ifndef snapinfo_h
#define snapinfo_h

/* The types of machine we can handle */

enum {

  MACHINE_48K   =0,
  MACHINE_128K  =1,
  MACHINE_PLUS2 =2,
  MACHINE_PLUS2A=3,
  MACHINE_PLUS3 =4

};

/* The various peripherals available */

enum {

  PERIPH_NONE  =0,
  PERIPH_IF1   =0x01, /* Interface 1 */
  PERIPH_MGT   =0x02, /* MGT (Disciple/+D) */
  PERIPH_SAMRAM=0x04  /* SamRam */

};

struct snapinfo {

  /* The things which directly influence the Spectrum */

  unsigned char a ,f ; unsigned short bc ,de ,hl ;
  unsigned char a_,f_; unsigned short bc_,de_,hl_;

  unsigned short ix,iy; unsigned char i,r;
  unsigned short pc,sp;

  unsigned char iff1,iff2,im;

  unsigned char outULA;	/* The last byte output to the ULA */

  unsigned char machine; /* The machine type the snap is intended for */
  /* Allowed values are the MACHINE_xxx defines above */

  unsigned short periph; /* Additional peripherals */
  /* OR the PERIPH_xxx defines above together */

  unsigned char out1ffd; /* Last out to #1ffd (.z80 v3x, .zxs) */
  unsigned char out7ffd; /* Last out to #7ffd (.z80 v2+, .zxs) */
  unsigned char outfffd; /* Last out to #fffd (.z80 v2+, .zxs) */
  unsigned char soundchip[16]; /* Soundchip registers (.z80 v2+, .zxs) */
  unsigned char ayactive; /* AY-3-8912 emulated in 48k mode? (.z80 v2+) */

  unsigned char keys[8]; /* Which keys are pressed (.zxs) */

  /* version is used by .z80 and .zxs files to know which version of the
     file is being processed

     compressed is used by .z80 v1 to signify whether the data is compressed,
     and by .zxs to signify the compression method used

     These don't need to be set if they are not relevant to the format
     being considered; they are never written out.

  */

  unsigned short version; /* which version of the format is this? */
  unsigned short compressed; /* Is the data compressed? */

  /* Things defined by emulators */

  unsigned char issue2;	/* Issue 2 emulation? (.z80, .zxs) */
  unsigned char rreg;	/* R register emulation (.z80 v2+) */
  unsigned char ldir;	/* LDIR emulation (.z80 v2+) */
  unsigned char doubleint; /* Double interrupt frequency?
							(.z80 pre-v3.05) */
  unsigned char videosync; /* Video synchronisation level (.z80) */
  unsigned char joystick; /* Joystick emulation (.z80) */
  unsigned int  samram;	/* SamRam latch status (.z80) */
			/* Note: this contains the SamRom bit from .z80 v1 */
  unsigned char if1; /* Is the IF1 ROM paged? (.z80 v2+) */
  unsigned char mgt; /* Is the MGT ROM paged? (.z80 v3+) */
  unsigned char mgttype; /* Type of MGT emulation (.z80 v3+) */
  unsigned char multiface; /* Is the Multiface ROM paged? (.z80 v3+) */
  unsigned char discbutton; /* Disciple inhibit button (.z80 v3+) */
  unsigned char discflag; /* Can we page the Disciple ROM? (.z80 v3+) */
  unsigned long tstates; /* T-states since last interrupt (.z80 v3+, .zxs) */
  unsigned char spectator; /* Flag byte used by Spectator (.z80 v3+) */
  unsigned char userstick[20]; /* User-defined joystick mappings
				  (.z80 v3+) */

  unsigned char flash;	/* Is flash ink or paper? (Spectrum v0.99) */
  unsigned char interrupt; /* Is an interrupt pending?
			      (Spectrum v0.99) */

};

void snapinfoDefault(struct snapinfo *p);

#endif				/* #ifndef snapinfo_h */
