/* z80.h: routines for dealing with .z80 format
   Copyright (c) 1997 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#ifndef z80_h
#define z80_h

#include <stdio.h>

#ifndef hardware_h
#include "hardware.h"
#endif

#ifndef snapinfo_h
#include "snapinfo.h"
#endif

/* the lengths of the .z80 format headers */

#define z80_v1_headerLength 30
#define z80_v2_headerLength 23
#define z80_v3_headerLength 54
#define z80_v3x_headerLength 55

#define z80_numPages 8	/* Max number of pages in a .z80 file */

#define Z80_DEFAULTOUTVERSION 3 /* The version to produce if not specified */

unsigned z80_compress(unsigned char *dest,const unsigned char *src,
		      unsigned Length);
unsigned z80_uncompress(unsigned char *dest,const unsigned char *src,
			unsigned Length);

int z80_readHeader(FILE *f,struct snapinfo *snap);
int z80_writev1Header(FILE *f,const struct snapinfo *snapptr,int v2);
int z80_writev2Header(FILE *f,const struct snapinfo *snapptr,int v3);
int z80_writev3Header(FILE *f,const struct snapinfo *snapptr);

int z80_readBlock(FILE *f,int *pptr,unsigned char **buffer,
		  unsigned *uncompLength);
int z80_readv1Data(FILE *f,unsigned char *buffer,int compressed);
int z80_writeBlock(FILE *f,int num,unsigned char *page,unsigned long length);
int z80_writev1Data(FILE *f,unsigned char *page5,unsigned char *page1,
		    unsigned char *page2);

int z80_parsePage(int page,unsigned char *buffer,unsigned length,
		  struct snapinfo *sptr,unsigned char **pages,
		  romData *romPages);
int z80_parseRom(unsigned char *buffer,unsigned length,int which,
		 romData *romPages);

/* The hardware maps */

extern hardwareMap z80_version1Map[]; /* Version 1 */
extern hardwareMap z80_version2Map[]; /* Version 2 */
extern hardwareMap z80_version3Map[]; /* Version 3 */

#endif
