/* zxs.c: routines for dealing with .zxs format files
   Copyright (c) 1998 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#ifndef zxs_h
#define zxs_h

#include <stdio.h>

#ifndef hardware_h
#include "hardware.h"
#endif

#ifndef snapinfo_h
#include "snapinfo.h"
#endif

extern int zxs_majorVersion;
extern int zxs_minorVersion;

extern int zxs_status;

typedef struct zxs_chunkList {
  char ID[5];
  struct zxs_chunkList *sublist;
  struct zxs_chunkList *next;
} zxs_chunkList;

extern zxs_chunkList *zxs_unknownChunks;
extern zxs_chunkList *zxs_shortChunks;
extern zxs_chunkList *zxs_longChunks;

int zxs_readChunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		  unsigned long *bytesread);
int zxs_readChunkHeader(FILE *f,char *ID,unsigned long *length);
int zxs_readParentChunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
			unsigned long length);
int zxs_readfmtzChunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readrZ80Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readr048Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readr128Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readrPlus3Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
			unsigned long length);
int zxs_readram0Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readram1Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readram2Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readram3Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readram4Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readram5Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readram6Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);
int zxs_readram7Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		      unsigned long length);

int zxs_writeChunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		   zxs_chunkList *list,unsigned long *length);
int zxs_writeRIFFChunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeLISTChunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writefmtzChunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writerZ80Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writer048Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writer128Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writerPlus3Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
			 zxs_chunkList *list,unsigned long *length);
int zxs_writeram0Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeram1Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeram2Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeram3Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeram4Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeram5Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeram6Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);
int zxs_writeram7Chunk(FILE *f,struct snapinfo *sptr,unsigned char **pages,
		       zxs_chunkList *list,unsigned long *length);

int zxs_addChunkToList(zxs_chunkList **list,const char *ID);
void zxs_cleanUp(void);
void zxs_deleteList(zxs_chunkList **list);

extern hardwareMap zxs_hardwareMap[]; /* The hardware map */

#endif		/* #ifndef zxs_h */
