/* snp.c: routines for dealing with .snp format files
   Copyright (c) 1997-8 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@cam.ac.uk (until at least June 1999)
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#include <stdio.h>
#include <string.h>

#include "snapconv.h"
#include "snapinfo.h"

#include "snp.h"

int snp_read(FILE *f,struct snapinfo *sptr,unsigned char *buffer)
{unsigned char Header[snp_headerSize];

 /* Read in the memory contents */
 if(fread(buffer,0xc000,1,f)!=1) return ERROR_READ;

 if(fread(Header,snp_headerSize,1,f)!=1) return ERROR_READ;

 sptr->f=Header[0]; sptr->a=Header[1];
 sptr->outULA=Header[2]&0x07;
 /* Header offset 3 is reserved */
 sptr->bc=Header[ 4]+Header[ 5]*0x100;
 sptr->de=Header[ 6]+Header[ 7]*0x100;
 sptr->hl=Header[ 8]+Header[ 9]*0x100;
 sptr->pc=Header[10]+Header[11]*0x100;
 sptr->sp=Header[12]+Header[13]*0x100;
 sptr->ix=Header[14]+Header[15]*0x100;
 sptr->iy=Header[16]+Header[17]*0x100;
 sptr->iff2=!!Header[18]; sptr->iff1=!!Header[19];
 sptr->im=Header[20];
 sptr->r=Header[21]; sptr->i=Header[22];
 sptr->f_=Header[23]; sptr->a_=Header[24];
 sptr->bc_=Header[25]+Header[26]*0x100;
 sptr->de_=Header[27]+Header[28]*0x100;
 sptr->hl_=Header[29]+Header[30]*0x100;

 if(verbose>=2)
   {
     fprintf(stderr,"AF =0x%02x%02x Border %d\n",sptr->a,sptr->f,sptr->outULA);
     fprintf(stderr,"BC =0x%04x DE=0x%04x\n",sptr->bc,sptr->de);
     fprintf(stderr,"HL =0x%04x PC=0x%04x\n",sptr->hl,sptr->pc);
     fprintf(stderr,"SP =0x%04x IX=0x%04x\n",sptr->sp,sptr->ix);
     fprintf(stderr,"IY =0x%04x\n",sptr->iy);
     fprintf(stderr,"IFF1=%d IFF2=%d IM %d\n",sptr->iff1,sptr->iff2,sptr->im);
     fprintf(stderr," R =0x%02x    I=0x%02x\n",sptr->r,sptr->i);
     fprintf(stderr,"AF'=0x%02x%02x BC'=0x%04x\n",sptr->a_,sptr->f_,sptr->bc_);
     fprintf(stderr,"DE'=0x%04x HL'=0x%04x\n",sptr->de_,sptr->hl_);
   }

 return ERROR_NONE;

}

int snp_write(FILE *f,struct snapinfo *sptr,unsigned char *buffer)
{
 /* write out memory */
 fwrite(buffer,0xc000,1,f);

 fputc(sptr->f,f); fputc(sptr->a,f);
 fputc( (sptr->outULA)&0x07 , f); fputc('\0',f); /* reserved */

 writeInt(sptr->bc,f); writeInt(sptr->de,f);
 writeInt(sptr->hl,f); writeInt(sptr->pc,f);
 writeInt(sptr->sp,f); writeInt(sptr->ix,f);
 writeInt(sptr->iy,f);

 fputc(sptr->iff2,f); fputc(sptr->iff1,f); fputc(sptr->im,f);
 fputc(sptr->r,f); fputc(sptr->i,f);

 fputc(sptr->f_,f); fputc(sptr->a_,f);
 writeInt(sptr->bc_,f); writeInt(sptr->de_,f);
 writeInt(sptr->hl_,f);

 return ferror(f) ? ERROR_WRITE : ERROR_NONE;

}
